/**
 *
 */
import { RecordState } from "./RecordState";
export class RecordSetState {
    constructor(value) {
        if (typeof value === 'string') {
            this._value = JSON.parse(value);
        }
        else {
            this._value = value;
        }
        if (!this._value.records) {
            throw new Error('Invalid record set -- missing records field');
        }
        if (!Array.isArray(this._value.records)) {
            throw new Error('Invalid record set -- records field is not an array');
        }
    }
    // --- State Management Helpers --- //
    static emptyRecordSet() {
        return new RecordSetState({
            defaultActionId: null,
            records: [],
            hasMore: false,
            type: "hxgn.api.dialog.RecordSet"
        });
    }
    // --- State Import/Export --- //
    copyAsJsonObject() {
        return JSON.parse(this.copyAsJsonString());
    }
    copyAsJsonString() {
        return JSON.stringify(this.internalValue());
    }
    internalValue() {
        return this._value;
    }
    // --- State Management --- //
    addAllRecords(recordSetState) {
        for (const r of recordSetState.internalValue().records) {
            this.addOrUpdateRecord(new RecordState(r));
        }
    }
    addOrUpdateRecord(recordState) {
        let found = false;
        const recordCopy = JSON.parse(JSON.stringify(recordState.internalValue()));
        for (const r of this.internalValue().records) {
            if (r.id === recordCopy.id) {
                for (const k of Object.keys(r)) {
                    r[k] = recordCopy[k];
                }
                found = true;
                break;
            }
        }
        if (!found) {
            this.internalValue().records.push(recordCopy);
        }
    }
    findRecordAtId(id) {
        for (const r of this.records()) {
            if (r.recordId() === id) {
                return r;
            }
        }
        return null;
    }
    *records() {
        let index = 0;
        while (index < this.internalValue().records.length) {
            yield new RecordState(this.internalValue().records[index++]);
        }
    }
    size() {
        return this.internalValue().records.length;
    }
}
