/**
 *
 */
export class RecordState {
    constructor(value) {
        if (typeof value === 'string') {
            this._value = JSON.parse(value);
        }
        else {
            this._value = value;
        }
        if (!this._value.id) {
            throw new Error('Invalid record -- missing id field');
        }
        if (typeof this._value.id !== 'string') {
            throw new Error('Invalid record set -- id field is not a string');
        }
        if (!this._value.properties) {
            throw new Error('Invalid record -- missing properties field');
        }
        if (!Array.isArray(this._value.properties)) {
            throw new Error('Invalid record set -- properties field is not an array');
        }
        if (!this._value.annotations) {
            throw new Error('Invalid record -- missing annotations field');
        }
        if (!Array.isArray(this._value.annotations)) {
            throw new Error('Invalid record set -- annotations field is not an array');
        }
    }
    // --- State Management Helpers --- //
    static getPropertyValue(record, propertyName) {
        return (new RecordState(record)).getPropertyValue(propertyName);
    }
    static setPropertyValue(record, propertyName, value) {
        (new RecordState(record)).setPropertyValue(propertyName, value);
    }
    // --- State Import/Export --- //
    copyAsJsonObject() {
        return JSON.parse(this.copyAsJsonString());
    }
    copyAsJsonString() {
        return JSON.stringify(this.internalValue());
    }
    internalValue() {
        return this._value;
    }
    // --- State Management --- //
    recordId() {
        return this.internalValue().id;
    }
    getPropertyValue(propertyName) {
        for (const p of this.internalValue().properties) {
            if (p.name === propertyName) {
                return p.value;
            }
        }
        return undefined;
    }
    setPropertyValue(propertyName, value) {
        let found = false;
        for (const p of this.internalValue().properties) {
            if (p.name === propertyName) {
                p.value = value;
                found = true;
                break;
            }
        }
        if (!found) {
            this.internalValue().properties.push({
                "name": propertyName,
                "format": null,
                "annotations": [],
                "type": "hxgn.api.dialog.Property",
                "value": value
            });
        }
    }
}
