import { Menu } from './Menu';
import { ViewType } from './types';
/**
 * A View represents a Catavolt 'Pane' definition.  A Pane can be thought of as a 'panel' or UI component
 * that is responsible for displaying a data record or records. The Pane describes 'how' and 'where' the data will be
 * displayed, as well as surrounding 'meta' data (i.e. the Pane title, the Pane's menus).  The Pane itself does not contain
 * the record or records to be displayed, but may be combined with a {@link Record}(s) to display the data.
 */
export declare abstract class View {
    readonly alias: string;
    readonly defaultActionId: string;
    readonly id: string;
    readonly name: string;
    readonly menu: Menu;
    readonly title: string;
    readonly type: ViewType;
    /**
     * Find the title for this Pane
     * @returns {string}
     */
    findTitle(): string;
    /**
     * Find a menu def on this View with the given actionId
     * @param actionId
     * @returns {Menu}
     */
    findMenuAt(actionId: string): Menu;
}
